@@ -24,7 +24,8 @@
 #endif
 #define KB_TO_BYTES(kb) ((kb) * 1024)
 #define MB_TO_BYTES(mb) ((mb) * 1024 * 1024)
-#define GB_TO_BYTES(gb) ((gb) * 1024 * 1024 * 1024ULL)
+#define GB_TO_BYTES(gb) ((uint64_t)(gb) * 1024 * 1024 * 1024ULL)
+#define TB_TO_BYTES(tb) ((uint64_t)(tb) * 1024 * 1024 * 1024 * 1024ULL)
 
 #define MS_TO_NS(ms) ((uint64_t)(ms) * 1000000)
 #define SEC_TO_NS(ms) ((uint64_t)(ms) * 1000000000)
@@ -150,6 +151,32 @@ extern const struct aws_byte_cursor g_delete_method;
 AWS_S3_API
 extern const uint32_t g_s3_max_num_upload_parts;
 
+AWS_S3_API
+/* TODO: now this is hard-coded as 8MB, but maybe something else is better. */
+extern const size_t g_streaming_buffer_size;
+
+AWS_S3_API
+extern const double g_default_throughput_target_gbps;
+
+AWS_S3_API
+extern const uint64_t g_streaming_object_size_threshold;
+
+AWS_S3_API
+extern const uint64_t g_default_part_size_fallback;
+
+AWS_S3_API
+extern const uint64_t g_default_max_part_size;
+
+AWS_S3_API
+extern const uint64_t g_s3_optimal_range_size_alignment;
+
+AWS_S3_API
+extern const uint32_t g_s3express_connection_limitation;
+AWS_S3_API
+extern const uint64_t g_s3express_connection_limitation_part_size_threshold;
+AWS_S3_API
+extern const uint64_t g_s3express_connection_limitation_object_size_threshold;
+
 /**
  * Returns AWS_S3_REQUEST_TYPE_UNKNOWN if name doesn't map to an enum value.
  */
@@ -230,12 +257,20 @@ void aws_s3_add_user_agent_header(struct aws_allocator *allocator, struct aws_ht
  * object-size. All output arguments are optional.*/
 AWS_S3_API
 int aws_s3_parse_content_range_response_header(
-    struct aws_allocator *allocator,
     struct aws_http_headers *response_headers,
     uint64_t *out_range_start,
     uint64_t *out_range_end,
     uint64_t *out_object_size);
 
+/* Given a Content-Range header value as a byte cursor, parses the range-start, range-end and
+ * object-size. All output arguments are optional. */
+AWS_S3_API
+int aws_s3_parse_content_range_cursor(
+    struct aws_byte_cursor content_range_cursor,
+    uint64_t *out_range_start,
+    uint64_t *out_range_end,
+    uint64_t *out_object_size);
+
 /* Given response headers, parses the content-length from a content-length response header.*/
 AWS_S3_API
 int aws_s3_parse_content_length_response_header(
@@ -308,6 +343,53 @@ int aws_s3_check_headers_for_checksum(
     struct aws_byte_buf *out_checksum_buffer,
     bool meta_request_level);
 
+/**
+ * Calculate client-level optimal range size based on memory and connection constraints.
+ * This function is called during client initialization to determine the base range size
+ * using the formula: MemoryLimit / concurrency / divisor.
+ * The result is rounded up to ensure proper alignment and applies minimum size constraints.
+ *
+ * @param memory_limit_in_bytes Total memory limit available for buffering
+ * @param max_connections Maximum number of concurrent connections
+ * @param out_client_optimal_range_size Output parameter for calculated client-level optimal range size
+ * @return AWS_OP_SUCCESS on success, AWS_OP_ERR on failure (caller should fall back to default)
+ */
+AWS_S3_API
+int aws_s3_calculate_client_optimal_range_size(
+    uint64_t memory_limit_in_bytes,
+    uint32_t max_connections,
+    uint64_t *out_client_optimal_range_size);
+
+/**
+ * Calculate request-level optimal range size by considering object-specific information.
+ * This function is called per request to adjust the client-level range size based on
+ * estimated object stored part size using: min(client_optimal_range_size, estimated_object_stored_part_size).
+ *
+ * @param client_optimal_range_size The client-level optimal range size from initialization
+ * @param estimated_object_stored_part_size Estimated size of object stored parts in S3
+ * @param is_express If the request is a s3express request or not.
+ * @param out_request_optimal_range_size Output parameter for calculated request-level optimal range size
+ * @return AWS_OP_SUCCESS on success, AWS_OP_ERR on failure (caller should fall back to client size)
+ */
+AWS_S3_API
+int aws_s3_calculate_request_optimal_range_size(
+    uint64_t client_optimal_range_size,
+    uint64_t estimated_object_stored_part_size,
+    bool is_express,
+    uint64_t *out_request_optimal_range_size);
+
+/**
+ * Extract the number of parts from an S3 ETag header value.
+ * S3 multipart upload ETags have the format "<hash>-<number_of_parts>".
+ * Single-part uploads have ETags without dashes.
+ *
+ * @param etag_header_value The ETag header value (may include quotes)
+ * @param out_num_parts Output parameter for the number of parts (1 for single-part uploads)
+ * @return AWS_OP_SUCCESS on success, AWS_OP_ERR on failure (invalid ETag format)
+ */
+AWS_S3_API
+int aws_s3_extract_parts_from_etag(struct aws_byte_cursor etag_header_value, uint32_t *out_num_parts);
+
 AWS_EXTERN_C_END
 
 #endif /* AWS_S3_UTIL_H */
